@@ -23,6 +23,7 @@
 #include <tqlayout.h>
 #include <tqlineedit.h>
 #include <tqpushbutton.h>
+#include <tqtimer.h>
 
 #include <dcopclient.h>
 
@@ -42,11 +43,14 @@
 #include <kgenericfactory.h>
 
 #include <unistd.h>
+#include <kpassdlg.h>
 #include <ksimpleconfig.h>
 #include <string>
 #include <stdio.h>
 #include <tqstring.h>
 
+#include <tdecryptographiccarddevice.h>
+
 #include "hwmanager.h"
 
 using namespace std;
@@ -83,6 +87,8 @@ TDEHWManager::TDEHWManager(TQWidget *parent, const char *name, const TQStringLis
 
 	base->deviceFilter->setListView(base->deviceTree);
 
+	deviceUpdateScheduled = false;
+
 	setRootOnlyMsg(i18n("<b>Device settings are system wide, and therefore require administrator access</b><br>To alter the system's device settings, click on the \"Administrator Mode\" button below."));
 	setUseRootOnlyMsg(true);
 
@@ -92,9 +98,8 @@ TDEHWManager::TDEHWManager(TQWidget *parent, const char *name, const TQStringLis
 	connect(base->showByConnection, TQT_SIGNAL(clicked()), TQT_SLOT(changed()));
 	connect(base->showByConnection, TQT_SIGNAL(clicked()), TQT_SLOT(populateTreeView()));
 
-	connect(hwdevices, TQT_SIGNAL(hardwareAdded(TDEGenericDevice*)), this, TQT_SLOT(populateTreeView()));
-	connect(hwdevices, TQT_SIGNAL(hardwareRemoved(TDEGenericDevice*)), this, TQT_SLOT(populateTreeView()));
-	connect(hwdevices, TQT_SIGNAL(hardwareUpdated(TDEGenericDevice*)), this, TQT_SLOT(deviceChanged(TDEGenericDevice*)));
+	connect(hwdevices, TQT_SIGNAL(hardwareAdded(TDEGenericDevice*)), this, TQT_SLOT(scheduleDeviceUpdate()));
+	connect(hwdevices, TQT_SIGNAL(hardwareRemoved(TDEGenericDevice*)), this, TQT_SLOT(scheduleDeviceUpdate()));
 
 	load();
 
@@ -103,6 +108,7 @@ TDEHWManager::TDEHWManager(TQWidget *parent, const char *name, const TQStringLis
 
 TDEHWManager::~TDEHWManager()
 {
+	TDEGlobal::hardwareDevices()->setTriggerlessHardwareUpdatesEnabled(false);
 	delete config;
 	delete systemconfig;
 }
@@ -127,8 +133,19 @@ void TDEHWManager::defaults()
 	load( true );
 }
 
+void TDEHWManager::scheduleDeviceUpdate()
+{
+	if (!deviceUpdateScheduled)
+	{
+		deviceUpdateScheduled = true;
+		TQTimer::singleShot(1000, this, TQT_SLOT(populateTreeView()));
+	}
+}
+
 void TDEHWManager::populateTreeView()
 {
+	deviceUpdateScheduled = false;
+
 	bool show_by_connection = base->showByConnection->isChecked();
 
 	// Figure out which device, if any, was selected
@@ -147,7 +164,13 @@ void TDEHWManager::populateTreeView()
 		TDEGenericHardwareList hwlist = hwdevices->listByDeviceClass(TDEGenericDeviceType::RootSystem);
 		TDEGenericDevice *hwdevice;
 		for ( hwdevice = hwlist.first(); hwdevice; hwdevice = hwlist.next() ) {
-			DeviceIconItem* item = new DeviceIconItem(base->deviceTree, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice);
+			if (hwdevice->type() == TDEGenericDeviceType::CryptographicCard) {
+				TDECryptographicCardDevice* cdevice = static_cast<TDECryptographicCardDevice*>(hwdevice);
+				connect(cdevice, SIGNAL(pinRequested(TQString,TDECryptographicCardDevice*)), this, SLOT(cryptographicCardPinRequested(TQString,TDECryptographicCardDevice*)));
+				cdevice->enableCardMonitoring(true);
+				cdevice->enablePINEntryCallbacks(true);
+			}
+			DeviceIconItem* item = new DeviceIconItem(base->deviceTree, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice->uniqueID());
 			if ((!selected_syspath.isNull()) && (hwdevice->systemPath() == selected_syspath)) {
 				base->deviceTree->ensureItemVisible(item);
 				base->deviceTree->setSelected(item, true);
@@ -159,11 +182,17 @@ void TDEHWManager::populateTreeView()
 		TDEHardwareDevices *hwdevices = TDEGlobal::hardwareDevices();
 		for (int i=0;i<=TDEGenericDeviceType::Last;i++) {
 			if (i != TDEGenericDeviceType::Root) {
-				DeviceIconItem* rootitem = new DeviceIconItem(base->deviceTree, hwdevices->getFriendlyDeviceTypeStringFromType((TDEGenericDeviceType::TDEGenericDeviceType)i), hwdevices->getDeviceTypeIconFromType((TDEGenericDeviceType::TDEGenericDeviceType)i, base->deviceTree->iconSize()), 0);
+				DeviceIconItem* rootitem = new DeviceIconItem(base->deviceTree, hwdevices->getFriendlyDeviceTypeStringFromType((TDEGenericDeviceType::TDEGenericDeviceType)i), hwdevices->getDeviceTypeIconFromType((TDEGenericDeviceType::TDEGenericDeviceType)i, base->deviceTree->iconSize()), TQString::null);
 				TDEGenericDevice *hwdevice;
 				TDEGenericHardwareList hwlist = hwdevices->listByDeviceClass((TDEGenericDeviceType::TDEGenericDeviceType)i);
 				for ( hwdevice = hwlist.first(); hwdevice; hwdevice = hwlist.next() ) {
-					DeviceIconItem* item = new DeviceIconItem(rootitem, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice);
+					if (hwdevice->type() == TDEGenericDeviceType::CryptographicCard) {
+						TDECryptographicCardDevice* cdevice = static_cast<TDECryptographicCardDevice*>(hwdevice);
+						connect(cdevice, SIGNAL(pinRequested(TQString,TDECryptographicCardDevice*)), this, SLOT(cryptographicCardPinRequested(TQString,TDECryptographicCardDevice*)));
+						cdevice->enableCardMonitoring(true);
+						cdevice->enablePINEntryCallbacks(true);
+					}
+					DeviceIconItem* item = new DeviceIconItem(rootitem, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice->uniqueID());
 					if ((!selected_syspath.isNull()) && (hwdevice->systemPath() == selected_syspath)) {
 						base->deviceTree->ensureItemVisible(item);
 						base->deviceTree->setSelected(item, true);
@@ -180,8 +209,14 @@ void TDEHWManager::populateTreeViewLeaf(DeviceIconItem *parent, bool show_by_con
 		TDEGenericHardwareList hwlist = hwdevices->listAllPhysicalDevices();
 		TDEGenericDevice *hwdevice;
 		for ( hwdevice = hwlist.first(); hwdevice; hwdevice = hwlist.next() ) {
+			if (hwdevice->type() == TDEGenericDeviceType::CryptographicCard) {
+				TDECryptographicCardDevice* cdevice = static_cast<TDECryptographicCardDevice*>(hwdevice);
+				connect(cdevice, SIGNAL(pinRequested(TQString,TDECryptographicCardDevice*)), this, SLOT(cryptographicCardPinRequested(TQString,TDECryptographicCardDevice*)));
+				cdevice->enableCardMonitoring(true);
+				cdevice->enablePINEntryCallbacks(true);
+			}
 			if (hwdevice->parentDevice() == parent->device()) {
-				DeviceIconItem* item = new DeviceIconItem(parent, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice);
+				DeviceIconItem* item = new DeviceIconItem(parent, hwdevice->detailedFriendlyName(), hwdevice->icon(base->deviceTree->iconSize()), hwdevice->uniqueID());
 				if ((!selected_syspath.isNull()) && (hwdevice->systemPath() == selected_syspath)) {
 					base->deviceTree->ensureItemVisible(item);
 					base->deviceTree->setSelected(item, true);
@@ -192,21 +227,14 @@ void TDEHWManager::populateTreeViewLeaf(DeviceIconItem *parent, bool show_by_con
 	}
 }
 
-void TDEHWManager::deviceChanged(TDEGenericDevice* device) {
-	TQListViewItemIterator it(base->deviceTree);
-	while (it.current()) {
-		DeviceIconItem* item = dynamic_cast<DeviceIconItem*>(it.current());
-		if (item) {
-			TDEGenericDevice* candidate = item->device();
-			if (candidate) {
-				if (candidate->systemPath() == device->systemPath()) {
-					if (item->text(0) != device->detailedFriendlyName()) {
-						item->setText(0, device->detailedFriendlyName());
-					}
-				}
-			}
-		}
-		++it;
+void TDEHWManager::cryptographicCardPinRequested(TQString prompt, TDECryptographicCardDevice* cdevice) {
+	TQString password;
+	int result = KPasswordDialog::getPassword(password, prompt);
+	if (result == KPasswordDialog::Accepted) {
+		cdevice->setProvidedPin(password);
+	}
+	else {
+		cdevice->setProvidedPin(TQString::null);
 	}
 }
 
