@@ -1,4 +1,4 @@
-/* This file is part of the KDE project
+/* This file is part of the TDE project
    Copyright (C) 2004 David Faure <faure@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -18,9 +18,11 @@
 */
 
 #include "trashimpl.h"
+#include "discspaceutil.h"
+#include "trash_constant.h"
+
 #include <tdelocale.h>
 #include <klargefile.h>
-#include <tdeio/global.h>
 #include <tdeio/renamedlg.h>
 #include <tdeio/job.h>
 #include <kdebug.h>
@@ -30,11 +32,10 @@
 #include <kstandarddirs.h>
 #include <tdeglobalsettings.h>
 #include <kmountpoint.h>
+#include <tdemessagebox.h>
 #include <tdefileitem.h>
 #include <tdeio/chmodjob.h>
 
-#include <dcopref.h>
-
 #include <tqapplication.h>
 #include <tqeventloop.h>
 #include <tqfile.h>
@@ -322,6 +323,9 @@ bool TrashImpl::deleteInfo( int trashId, const TQString& fileId )
 bool TrashImpl::moveToTrash( const TQString& origPath, int trashId, const TQString& fileId )
 {
     kdDebug() << k_funcinfo << endl;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
     const TQString dest = filesPath( trashId, fileId );
     if ( !move( origPath, dest ) ) {
         // Maybe the move failed due to no permissions to delete source.
@@ -387,6 +391,9 @@ void TrashImpl::jobFinished(TDEIO::Job* job)
 bool TrashImpl::copyToTrash( const TQString& origPath, int trashId, const TQString& fileId )
 {
     kdDebug() << k_funcinfo << endl;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
     const TQString dest = filesPath( trashId, fileId );
     if ( !copy( origPath, dest ) )
         return false;
@@ -957,4 +964,202 @@ bool TrashImpl::parseURL( const KURL& url, int& trashId, TQString& fileId, TQStr
     return true;
 }
 
+bool TrashImpl::adaptTrashSize( const TQString& origPath, int trashId )
+{
+    TDEConfig config( "trashrc" );
+
+    const TQString trashPath = trashDirectoryPath( trashId );
+    config.setGroup( trashPath );
+
+    bool useTimeLimit = config.readBoolEntry( "UseTimeLimit", false );
+    bool useSizeLimit = config.readBoolEntry( "UseSizeLimit", true );
+		int  sizeLimitType = config.readNumEntry( "SizeLimitType", TrashConstant::SIZE_LIMIT_PERCENT );
+    double percent = config.readDoubleNumEntry( "Percent", 10 );
+		double fixedSize = config.readDoubleNumEntry( "FixedSize", 500 );
+		int fixedSizeUnit = config.readNumEntry( "FixedSizeUnit", TrashConstant::SIZE_ID_MB );
+    int actionType = config.readNumEntry( "LimitReachedAction", TrashConstant::ACTION_WARNING );
+
+    if ( useTimeLimit ) { // delete all files in trash older than X days
+        const int maxDays = config.readNumEntry( "Days", 32000 );
+        const TQDateTime currentDate = TQDateTime::currentDateTime();
+
+        const TrashedFileInfoList trashedFiles = list();
+        for ( uint i = 0; i < trashedFiles.count(); ++i ) {
+            struct TrashedFileInfo info = trashedFiles[ i ];
+            if ( info.trashId != trashId )
+                continue;
+
+            if ( info.deletionDate.daysTo( currentDate ) > maxDays )
+              del( info.trashId, info.fileId );
+        }
+    }
+
+    if ( useSizeLimit ) { // check if size limit exceeded
+        // calculate size of the files to be put into the trash
+        unsigned long additionalSize = DiscSpaceUtil::sizeOfPath( origPath );
+        TQString trashPathName = trashPath + "/files/";
+        DiscSpaceUtil util( trashPathName );
+        unsigned long requiredTrashSpace = util.sizeOfPath(trashPathName) + additionalSize;
+        unsigned long trashLimit = 0;
+        if ( sizeLimitType == TrashConstant::SIZE_LIMIT_PERCENT)
+        {
+        	trashLimit = (unsigned long)(1024 * percent * util.size() / 100.0);
+        }
+        else if ( sizeLimitType == TrashConstant::SIZE_LIMIT_FIXED)
+        {
+        	double trashLimitTemp = fixedSize;
+        	while (fixedSizeUnit > TrashConstant::SIZE_ID_B)
+					{
+						trashLimitTemp *= 1024;
+						--fixedSizeUnit;
+					}
+	        trashLimit = (unsigned long)trashLimitTemp;
+        }
+        if ( additionalSize > trashLimit ) {
+					m_lastErrorCode = TDEIO::ERR_SLAVE_DEFINED;
+					m_lastErrorMessage = i18n( "The file '%1' is bigger than the '%2' trash bin size.\n"
+					                           "It cannot be trashed." ).arg(origPath).arg(util.mountPoint());
+					return false;
+        } else if ( requiredTrashSpace > trashLimit ) {
+            if ( actionType == TrashConstant::ACTION_WARNING ) { // warn the user only
+                m_lastErrorCode = TDEIO::ERR_SLAVE_DEFINED;
+                m_lastErrorMessage = i18n( "There is not enough space left in trash folder '%1'.\n"
+                                           "The file cannot be trashed. Clean the trash manually and try again.")
+                                           .arg(util.mountPoint());
+                return false;
+            } else {
+                TQDir dir( trashPath + "/files" );
+                const TQFileInfoList *infos = 0;
+                if ( actionType == TrashConstant::ACTION_DELETE_OLDEST )  // delete oldest files first
+                    infos = dir.entryInfoList( TQDir::Files | TQDir::Dirs, TQDir::Time | TQDir::Reversed );
+                else if ( actionType == TrashConstant::ACTION_DELETE_BIGGEST ) // delete biggest files first
+                    infos = dir.entryInfoList( TQDir::Files | TQDir::Dirs, TQDir::Size );
+                else {
+                    tqWarning( "<TrashImpl::adaptTrashSize> Should never happen!" );
+                    return false;
+                }
+
+                TQFileInfoListIterator it( *infos );
+                TQFileInfo *info;
+                bool deleteFurther = true;
+                while ( ((info = it.current()) != 0) && deleteFurther ) {
+                    if ( info->fileName() != "." && info->fileName() != ".." ) {
+                        del( trashId, info->fileName() ); // delete trashed file
+                        if ( (util.sizeOfPath(trashPathName) + additionalSize) < trashLimit ) // check whether we have enough space now
+                            deleteFurther = false;
+                    }
+                    ++it;
+                }
+            }
+        }
+    }
+
+    return true;
+}
+
+void TrashImpl::resizeTrash(int trashId)
+{
+	TDEConfig config("trashrc");
+	const TQString trashPath = trashDirectoryPath(trashId);
+	config.setGroup(trashPath);
+
+	bool useTimeLimit = config.readBoolEntry("UseTimeLimit", false);
+	bool useSizeLimit = config.readBoolEntry("UseSizeLimit", true);
+	int  sizeLimitType = config.readNumEntry("SizeLimitType", TrashConstant::SIZE_LIMIT_PERCENT);
+	double percent = config.readDoubleNumEntry("Percent", 10);
+	double fixedSize = config.readDoubleNumEntry("FixedSize", 500);
+	int fixedSizeUnit = config.readNumEntry("FixedSizeUnit", TrashConstant::SIZE_ID_MB);
+	int actionType = config.readNumEntry("LimitReachedAction", 0);
+
+	if (useTimeLimit)
+	{
+		// delete all files in trash older than X days
+		const int maxDays = config.readNumEntry("Days", 32000);
+		const TQDateTime currentDate = TQDateTime::currentDateTime();
+		const TrashedFileInfoList trashedFiles = list();
+		for (uint i = 0; i < trashedFiles.count(); ++i)
+		{
+			struct TrashedFileInfo info = trashedFiles[ i ];
+			if (info.trashId != trashId)
+			{
+				continue;
+			}
+			if (info.deletionDate.daysTo(currentDate) > maxDays)
+			{
+				del(info.trashId, info.fileId);
+			}
+		}
+	}
+
+	if (useSizeLimit)
+	{
+		// check if size limit exceeded
+		TQString trashPathName = trashPath + "/files/";
+		DiscSpaceUtil util(trashPathName);
+		unsigned long currTrashSize = util.sizeOfPath(trashPathName);
+		unsigned long trashLimit = 0;
+		if (sizeLimitType == TrashConstant::SIZE_LIMIT_PERCENT)
+		{
+			trashLimit = (unsigned long)(1024 * percent * util.size() / 100.0);
+		}
+		else if (sizeLimitType == TrashConstant::SIZE_LIMIT_FIXED)
+		{
+			double trashLimitTemp = fixedSize;
+			while (fixedSizeUnit > TrashConstant::SIZE_ID_B)
+			{
+				trashLimitTemp *= 1024;
+				--fixedSizeUnit;
+			}
+			trashLimit = (unsigned long)trashLimitTemp;
+		}
+		if (currTrashSize > trashLimit)
+		{
+			if (actionType == TrashConstant::ACTION_WARNING)
+			{
+				// warn the user only
+        KMessageBox::error(0, i18n("The current size of trash folder '%1' is bigger than the allowed size.\n"
+																	 "Clean the trash manually.").arg(util.mountPoint()));
+				return;
+			}
+			else
+			{
+				TQDir dir(trashPath + "/files");
+				const TQFileInfoList *infos = 0;
+				if (actionType == TrashConstant::ACTION_DELETE_OLDEST)
+				{
+					// delete oldest files first
+					infos = dir.entryInfoList(TQDir::Files | TQDir::Dirs, TQDir::Time | TQDir::Reversed);
+				}
+				else if (actionType == TrashConstant::ACTION_DELETE_BIGGEST)
+				{
+					// delete biggest files first
+					infos = dir.entryInfoList(TQDir::Files | TQDir::Dirs, TQDir::Size);
+				}
+				else
+				{
+					tqWarning("<TrashImpl::resizeTrash> Should never happen!");
+					return;
+				}
+
+				TQFileInfoListIterator it(*infos);
+				TQFileInfo *info;
+				bool deleteFurther = true;
+				while (((info = it.current()) != 0) && deleteFurther)
+				{
+					if (info->fileName() != "." && info->fileName() != "..")
+					{
+						del(trashId, info->fileName()); // delete trashed file
+						if ((util.sizeOfPath(trashPathName)) < trashLimit)
+						{
+							// check whether we have enough space now
+							deleteFurther = false;
+						}
+					}
+					++it;
+				}
+			}
+		}
+	}
+}
+
 #include "trashimpl.moc"
